import { Component, Input, Output, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
export const LY_CHECKBOX_CONTROL_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => LyCheckbox),
    multi: true
};
export var CheckboxState;
(function (CheckboxState) {
    /** The initial state. */
    CheckboxState[CheckboxState["Init"] = 0] = "Init";
    /** Checked state. */
    CheckboxState[CheckboxState["Checked"] = 1] = "Checked";
    /** Unchecked state. */
    CheckboxState[CheckboxState["Unchecked"] = 2] = "Unchecked";
    /** Indeterminate state. */
    CheckboxState[CheckboxState["Indeterminate"] = 3] = "Indeterminate";
})(CheckboxState || (CheckboxState = {}));
export class LyCheckbox {
    constructor() {
        this._currentCheckState = CheckboxState.Init;
        this.checked = false;
        this.change = new EventEmitter();
        this.indeterminateChange = new EventEmitter();
        this.onTouched = () => { };
        this._controlValueAccessorChangeFn = (value) => { };
    }
    /** Toggles the `checked` state of the checkbox. */
    toggle() {
        this.checked = !this.checked;
    }
    _onChangeEvent(event) {
        event.stopPropagation();
    }
    _onInputClick(event) {
        const ev = event.target;
        this.checked = ev.checked;
        this._controlValueAccessorChangeFn(this.checked);
        console.log('_onInputClick', ev.checked);
    }
    writeValue(value) {
        this.checked = !!value;
        console.log('value', this.checked);
    }
    registerOnChange(fn) {
        this._controlValueAccessorChangeFn = fn;
    }
    ngOnChanges(changes) {
        console.log('changes', changes);
        if (changes['src']) {
            // fn
        }
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    ngOnInit() {
    }
    ngAfterContentInit() {
    }
    ngOnDestroy() {
    }
}
LyCheckbox.decorators = [
    { type: Component, args: [{
                selector: 'ly-checkbox',
                template: "<label> <div class=\"ly-checkbox-container\"> <input #input [hidden]=\"false\" type=\"checkbox\" [id]=\"id\" [required]=\"required\" [checked]=\"checked\" [value]=\"value\" [disabled]=\"disabled\" [name]=\"name\" [indeterminate]=\"indeterminate\" [attr.aria-label]=\"ariaLabel\" [attr.aria-labelledby]=\"ariaLabelledby\" (change)=\"_onChangeEvent($event)\" (click)=\"_onInputClick($event)\"> </div> </label> ",
                styles: [":host { display: inline-block; } "],
                providers: [LY_CHECKBOX_CONTROL_VALUE_ACCESSOR]
            },] },
];
/** @nocollapse */
LyCheckbox.ctorParameters = () => [];
LyCheckbox.propDecorators = {
    "id": [{ type: Input, args: ['id',] },],
    "ariaLabel": [{ type: Input, args: ['attr.aria-label',] },],
    "ariaLabelledby": [{ type: Input, args: ['attr.aria-labelledby',] },],
    "required": [{ type: Input, args: ['required',] },],
    "labelPosition": [{ type: Input, args: ['labelPosition',] },],
    "color": [{ type: Input, args: ['color',] },],
    "value": [{ type: Input, args: ['value',] },],
    "name": [{ type: Input, args: ['name',] },],
    "disabled": [{ type: Input, args: ['disabled',] },],
    "checked": [{ type: Input, args: ['checked',] },],
    "indeterminate": [{ type: Input, args: ['indeterminate',] },],
    "change": [{ type: Output, args: ['change',] },],
    "indeterminateChange": [{ type: Output, args: ['indeterminateChange',] },],
};
//# sourceMappingURL=ly-checkbox.component.js.map