(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('rxjs/BehaviorSubject'), require('@angular/platform-browser'), require('chroma-js'), require('rxjs/Subject'), require('rxjs/add/operator/debounceTime')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', 'rxjs/BehaviorSubject', '@angular/platform-browser', 'chroma-js', 'rxjs/Subject', 'rxjs/add/operator/debounceTime'], factory) :
	(factory((global.alyleUi = global.alyleUi || {}, global.alyleUi.core = {}),global.ng.core,global.ng.common,global.Rx,global.ng.platformBrowser,global.chroma,global.Rx));
}(this, (function (exports,core,common,BehaviorSubject,platformBrowser,_chroma,Subject) { 'use strict';

var defaultTheme = {
    primary: 'blue',
    accent: 'pink',
    other: 'red',
    colorScheme: 'light',
    /** TODO: Deprecated */
    typography: {
        fontFamily: '\'Roboto\', sans-serif',
        fontSize: 14
    },
    variables: {
        typography: {
            fontFamily: '\'Roboto\', sans-serif',
            fontSize: 14
        }
    },
    schemes: {
        light: {
            colorText: 'rgba(0, 0, 0, 0.61)',
            disabled: 'rgba(0, 0, 0, 0.31)',
            bgText: '#f9f9f9',
            main: '#fff',
            colorShadow: '#777777',
            input: {
                label: 'rgba(0, 0, 0, 0.4)',
                underline: 'rgba(0, 0, 0, 0.11)'
            },
        },
        dark: {
            colorText: '#fff',
            disabled: 'rgba(255, 255, 255, 0.31)',
            bgText: '#303030',
            main: '#383838',
            colorShadow: '#777777',
            input: {
                label: 'rgba(255, 255, 255, 0.4)',
                underline: 'rgba(255, 255, 255, 0.11)'
            }
        }
    },
    palette: {
        'blue': {
            default: '#2196F3',
            contrast: 'light'
        },
        'pink': {
            default: '#ff4b73',
            contrast: 'light'
        },
        'red': {
            default: '#FF5252',
            contrast: 'light'
        }
    }
};

var AlyleServiceConfig = /** @class */ (function () {
    function AlyleServiceConfig() {
    }
    return AlyleServiceConfig;
}());

var hexToRgb = function (hex) {
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex), _a = result.map(function (val) { return parseInt(val, 16); }), r = _a[1], g = _a[2], b = _a[3];
    return result ? { r: r, g: g, b: b } : null;
};
var splitSliceJoin = function (string, start, end) { return string.split('').slice(start, end).join(''); };
// if hex and defined as #fff then convert it to standard 7 letter format #ffffff
var fixedHexFormat = function (arr) {
    return arr.map(function (c) {
        if (c.length === 4) {
            return "#" + (c[1] + c[1] + c[2] + c[2] + c[3] + c[3]);
        }
        else if (c.length === 7) {
            return c;
        }
    });
};
// get r,g,b,h,s and l with Bezier interpolation
// https://www.cl.cam.ac.uk/teaching/2000/AGraphHCI/SMEG/node3.html
// Check issue #3 for more info
var propBezInterpolate = function (charArr) {
    return function (colArr) {
        return function (x) {
            var y = 1 - x, v;
            return charArr.map(function (c) {
                if (colArr.length == 2) {
                    v = (y * colArr[0][c]) + (x * colArr[1][c]);
                }
                else if (colArr.length == 3) {
                    v = ((Math.pow(y, 2)) * colArr[0][c]) + (2 * y * x * colArr[1][c]) + ((Math.pow(x, 2)) * colArr[2][c]);
                }
                else if (colArr.length == 4) {
                    v = ((Math.pow(y, 3)) * colArr[0][c]) + (3 * (Math.pow(y, 2)) * x * colArr[1][c]) + (3 * y * (Math.pow(x, 2)) * colArr[2][c]) + ((Math.pow(x, 3)) * colArr[3][c]);
                }
                return Math.trunc(v);
            });
        };
    };
};
var extractHEX = function (arr) { return fixedHexFormat(arr).map(function (c) { return hexToRgb(c); }); };
var extractRGB = function (arr) {
    return arr.map(function (c) {
        var _a = splitSliceJoin(c, 4, -1).split(','), r = _a[0], g = _a[1], b = _a[2];
        return { r: r, g: g, b: b };
    });
};
var extractHSL = function (arr) {
    return arr.map(function (c) {
        c = splitSliceJoin(c, 4, -1).split(',');
        var h = c[0], s = splitSliceJoin(c[1], 0, -1), l = splitSliceJoin(c[2], 0, -1);
        return { h: h, s: s, l: l };
    });
};
var returnRGBStr = function (arr) { return "rgb(" + arr[0] + ", " + arr[1] + ", " + arr[2] + ")"; };
var returnHSLStr = function (arr) { return "hsl(" + arr[0] + ", " + arr[1] + "%, " + arr[2] + "%)"; };

var defaultOptions = {
    // input color options: hex, rgb or hsl
    inputFormat: 'hex',
    // number of color stops (cannot be less than colorArray.length)
    stops: 5,
    // input color array
    colorArray: ['#fff', '#000']
};

function gradStop(options) {
    options = Object.assign({}, defaultOptions, options);
    if (options.stops < options.colorArray.length) {
        throw new Error('Number of stops cannot be less than colorArray.length');
    }
    return computeStops(options);
}
// computeStops
var computeStops = function (options) {
    var outputArray = [];
    var init = function (opts) {
        switch (options.inputFormat) {
            case 'hex':
                return extractHEX(opts.colorArray);
            case 'rgb':
                return extractRGB(opts.colorArray);
            case 'hsl':
                return extractHSL(opts.colorArray);
        }
    };
    var stopsGenerator = function (opts) {
        var colorArray = opts.colorArray;
        var inc = 1.0 / (opts.stops - 1);
        var t = 0;
        for (var i = 0; i < opts.stops; i++) {
            if (opts.inputFormat === 'hex' || options.inputFormat === 'rgb') {
                var _a = propBezInterpolate(['r', 'g', 'b'])(colorArray)(t), r = _a[0], g = _a[1], b = _a[2];
                outputArray.push(returnRGBStr([r, g, b]));
            }
            else if (opts.inputFormat === 'hsl') {
                var _b = propBezInterpolate(['h', 's', 'l'])(colorArray)(t), h = _b[0], s = _b[1], l = _b[2];
                outputArray.push(returnHSLStr([h, s, l]));
            }
            t += inc;
        }
    };
    options.colorArray = init(options);
    stopsGenerator(options);
    return outputArray;
};

var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var classId = 0;
var themeId = 1111;
var ThemeColor = /** @class */ (function () {
    function ThemeColor() {
    }
    return ThemeColor;
}());
/**
 * DEPRECATED
 */
function themeProperty(color) {
    return color === 'primary' || color === 'accent' || color === 'other';
}
var LyTheme = /** @class */ (function () {
    function LyTheme(config, document, sanitizer, rendererFactory) {
        this.document = document;
        this.sanitizer = sanitizer;
        this.rendererFactory = rendererFactory;
        this._styleMap = new Map();
        /**
           * OBSOLETO
           */
        this.colors = [
            {
                name: 'pink',
                color: { '500': '#ff4b73' },
                contrast: 'light'
            },
            {
                name: 'pinkLight',
                color: { '500': '#f50057' },
                contrast: 'light'
            },
            {
                name: 'cyan',
                color: { '500': '#00bcd4' },
                contrast: 'light'
            },
            {
                name: 'red',
                color: { '500': '#FF5252' },
                contrast: 'light'
            },
            {
                name: 'amber',
                color: { '500': '#ffc107' },
                contrast: 'dark'
            },
            {
                name: 'teal',
                color: { '500': '#009688' },
                contrast: 'light'
            },
            {
                name: 'purple',
                color: { '500': '#ce30c9' },
                contrast: 'light'
            },
            {
                name: 'lightBlue',
                color: { '500': '#03A9F4' },
                contrast: 'light'
            },
            {
                name: 'blue',
                color: { '500': '#2196F3' },
                contrast: 'light'
            },
            {
                name: 'deepOrange',
                color: { '500': '#FF5722' },
                contrast: 'light'
            },
        ];
        this.renderer = this.rendererFactory.createRenderer(null, null);
        this.Id = "\u0138-" + themeId.toString(36);
        config = mergeDeep(defaultTheme, config);
        var primary = this._setColorPalette(config.primary, config.palette);
        var accent = this._setColorPalette(config.accent, config.palette);
        var other = this._setColorPalette(config.other, config.palette);
        var shade = config.shade;
        var scheme = config.schemes[config.colorScheme];
        var typography = config.typography;
        var variables = config.variables;
        // if (config.palette) {
        //   if (config.palette[config.primary]) {
        //     primary = config.palette[config.primary];
        //   }
        //   if (config.palette[config.accent]) {
        //     accent = config.palette[config.accent];
        //   }
        //   if (config.palette[config.other]) {
        //     other = config.palette[config.other];
        //   }
        // }
        var getAllColors = __assign({ primary: primary,
            accent: accent,
            other: other }, variables
        // colorText: {color: scheme.text.default},
        // bgText: {color: scheme.background.default}
        );
        getAllColors = mergeDeep(getAllColors, scheme);
        // this.createShades(primary.color[shade]);
        /* tslint:disable */
        this.AlyleUI = {
            currentTheme: config,
            palette: parsePalette(getAllColors),
        };
        /* tslint:enable */
        this.primary = new BehaviorSubject.BehaviorSubject(primary);
        this.accent = new BehaviorSubject.BehaviorSubject(accent);
        this.other = new BehaviorSubject.BehaviorSubject(other);
        this.scheme = new BehaviorSubject.BehaviorSubject(scheme);
        this.typography = new BehaviorSubject.BehaviorSubject(typography);
        this.shade = new BehaviorSubject.BehaviorSubject(shade);
        this.palette = new BehaviorSubject.BehaviorSubject(getAllColors);
        themeId++;
    }
    LyTheme.prototype.findColor = function (data) {
        var colors = this.colors.find(function (_) { return _.name === data; });
        if (colors) {
            return colors;
        }
        else {
            return new ThemeColor();
        }
    };
    LyTheme.prototype.sanitizerStyle = function (val) {
        return this.sanitizer.bypassSecurityTrustStyle(val);
    };
    LyTheme.prototype._setColorPalette = function (key, palette) {
        var colors = palette[key];
        if (colors) {
            if (Object.keys(colors).length <= 3) {
                /**get color of first item  */
                var shades = this.createShades(colors[(Object.keys(colors)[0])]);
                colors = mergeDeep(colors, shades);
            }
            return colors;
        }
        else {
            throw new Error(key + " not found in palette");
        }
    };
    /** get class name of color */
    /** get class name of color */
    LyTheme.prototype.getClassKey = /** get class name of color */
    function (color, of) {
        return this.Id + "-" + color.replace(':', '__') + "-" + of;
    };
    // private addShades(primary, accent, other) {
    //   this.createShades(primary.color[shade]);
    // }
    // private addShades(primary, accent, other) {
    //   this.createShades(primary.color[shade]);
    // }
    LyTheme.prototype._gradStop = 
    // private addShades(primary, accent, other) {
    //   this.createShades(primary.color[shade]);
    // }
    function (colors, stops) {
        return gradStop({
            stops: stops,
            inputFormat: 'hex',
            colorArray: colors
        });
    };
    LyTheme.prototype._getGrad = function (color) {
        var toBlack = this._gradStop(['#ffffff', color], 11);
        var toWhite = this._gradStop([color, '#000000'], 33);
        toBlack.pop();
        return toBlack.concat(toWhite);
    };
    LyTheme.prototype.createShades = function (color) {
        var ar = this._getGrad(color);
        var shades = {};
        ar.forEach(function (a, b) {
            var shadeId = "" + b * 100 / 2;
            if (b <= 20) {
                shades[shadeId] = a;
            }
        });
        return shades;
    };
    LyTheme.prototype.setTheme = function (config) {
        var currentTheme = this.AlyleUI.currentTheme;
        config = mergeDeep(currentTheme, config);
        if (config) {
            var primary = this._setColorPalette(config.primary, config.palette);
            var accent = this._setColorPalette(config.accent, config.palette);
            var other = this._setColorPalette(config.other, config.palette);
            var shade = config.shade;
            var scheme = config.schemes[config.colorScheme];
            var typography = config.typography;
            var variables = config.variables;
            var getAllColors = __assign({ primary: primary,
                accent: accent,
                other: other }, variables);
            getAllColors = mergeDeep(getAllColors, scheme);
            // this.createShades(primary.color[shade]);
            /* tslint:disable */
            this.AlyleUI = {
                currentTheme: config,
                palette: parsePalette(getAllColors)
            };
            /* tslint:enable */
            this.primary.next(primary);
            this.accent.next(accent);
            this.other.next(other);
            this.scheme.next(scheme);
            this.typography.next(typography);
            this.shade.next(shade);
            this.palette.next(getAllColors);
            this.updateOthersStyles();
        }
    };
    LyTheme.prototype.color = function (color, colors, shade) {
        console.warn('DEPRECATED');
        var $shade = shade ? shade : this.AlyleUI.currentTheme.shade;
        var result;
        result = this.getColorv2(color, colors, $shade);
        return result;
    };
    /**
     * get color of `string` in palette
     * @param value
     */
    /**
       * get color of `string` in palette
       * @param value
       */
    LyTheme.prototype.colorOf = /**
       * get color of `string` in palette
       * @param value
       */
    function (value) {
        var theme = this.AlyleUI.palette;
        // const shade = this.AlyleUI.currentTheme.shade;
        var current = theme;
        var values = value.split(/:/g);
        values.forEach(function (item, index) {
            if (current[item]) {
                current = current[item];
            }
            else if (index > 0) {
                console.warn("\n\n>>>`" + value + "` undefined in LyTheme\n");
            }
        });
        if (!current['default']) {
            current = value;
        }
        else {
            current = current['default'];
        }
        return current;
    };
    LyTheme.prototype.getColorv2 = function (colorName, colors, shade) {
        var ar = colors ? colors : this.AlyleUI.palette;
        if (ar[colorName]) {
            if (typeof ar[colorName].color === 'string' || typeof ar[colorName] === 'string') {
                return ar[colorName].color || ar[colorName];
            }
            else {
                return ar[colorName].color[shade];
            }
        }
        else {
            return colorName;
        }
    };
    LyTheme.prototype.paletteOf = function (colorName) {
        return this.findColor(colorName);
    };
    LyTheme.prototype.createStyle = function (_id, fn) {
        var arg = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            arg[_i - 2] = arguments[_i];
        }
        var styleData;
        var key = createKeyOf("" + this.Id + _id);
        // console.log('existStyle', this.existStyle(key));
        if (!this._styleMap.has(key)) {
            this._styleMap.set(key, null);
            classId++;
            var id = "ly_" + classId.toString(36);
            var styleContainer = this.renderer.createElement('style');
            var styleContent = this.renderer.createText("." + id + "{" + fn.apply(void 0, arg) + "}");
            this.renderer.setAttribute(styleContainer, "aui-id", key);
            this.renderer.appendChild(styleContainer, styleContent);
            this.renderer.appendChild(this.document.head, styleContainer);
            styleData = {
                id: id,
                key: key,
                styleContainer: styleContainer,
                styleContent: styleContent,
                value: {
                    fn: fn,
                    arg: arg
                }
            };
            this._styleMap.set(key, styleData);
            return styleData;
        }
        else {
            return this._styleMap.get(key);
        }
    };
    /** #style */
    /** #style */
    LyTheme.prototype.createClassContent = /** #style */
    function (value, id) {
        return "." + id + "{" + value + "}";
    };
    /** #style */
    /** #style */
    LyTheme.prototype.createStyleContent = /** #style */
    function (styleData) {
        return this.renderer.createText("." + styleData.id + "{" + (_a = styleData.value).fn.apply(_a, styleData.value.arg) + "}");
        var _a;
    };
    /** Update style of StyleData */
    /** Update style of StyleData */
    LyTheme.prototype.updateStyleValue = /** Update style of StyleData */
    function (style, styleText) {
        var styleContent = styleText;
        this.renderer.removeChild(style.styleContainer, style.styleContent);
        this.renderer.appendChild(style.styleContainer, styleContent);
        this._styleMap.set(style.key, Object.assign({}, style, {
            styleContainer: style.styleContainer,
            styleContent: styleContent
        }));
    };
    /** #style: Update all styles */
    /** #style: Update all styles */
    LyTheme.prototype.updateOthersStyles = /** #style: Update all styles */
    function () {
        var _this = this;
        this._styleMap.forEach(function (styleData) {
            var newStyleValue = _this.createStyleContent(styleData);
            _this.updateStyleValue(styleData, newStyleValue);
        });
    };
    /** Replace old class by newClass */
    /** Replace old class by newClass */
    LyTheme.prototype.updateClass = /** Replace old class by newClass */
    function (elementRef, renderer, newClass, oldClass) {
        if (oldClass) {
            renderer.removeClass(elementRef.nativeElement, oldClass);
        }
        renderer.addClass(elementRef.nativeElement, newClass);
    };
    LyTheme.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    LyTheme.ctorParameters = function () { return [
        { type: AlyleServiceConfig, decorators: [{ type: core.Optional },] },
        { type: undefined, decorators: [{ type: core.Inject, args: [common.DOCUMENT,] },] },
        { type: platformBrowser.DomSanitizer, },
        { type: core.RendererFactory2, },
    ]; };
    return LyTheme;
}());
function isObject(item) {
    return (item && typeof item === 'object' && !Array.isArray(item));
}
function mergeDeep(target, source) {
    var output = Object.assign({}, target);
    if (isObject(target) && isObject(source)) {
        Object.keys(source).forEach(function (key) {
            if (isObject(source[key])) {
                if (!(key in target)) {
                    Object.assign(output, (_a = {}, _a[key] = source[key], _a));
                }
                else {
                    output[key] = mergeDeep(target[key], source[key]);
                }
            }
            else {
                Object.assign(output, (_b = {}, _b[key] = source[key], _b));
            }
            var _a, _b;
        });
    }
    return output;
}
function parsePalette(palette) {
    var iterate = function (obj, keyObject) {
        for (var key in obj) {
            if (obj.hasOwnProperty(key)) {
                var key$ = keyObject ? keyObject + "__" + key : key;
                if (obj[key] !== null &&
                    typeof obj[key] === 'string' &&
                    key !== 'default') {
                    obj[key] = { default: obj[key] };
                }
                if (typeof obj[key] === 'object') {
                    iterate(obj[key], key$);
                }
            }
        }
        return obj;
    };
    return iterate(Object.assign({}, palette));
}
function createKeyOf(str) {
    return str.split('').map(function (char) {
        return char.charCodeAt(0).toString(36);
    }).join('·');
}

function getContrastYIQ(hexcolor) {
    var r = parseInt(hexcolor.substr(0, 2), 16);
    var g = parseInt(hexcolor.substr(2, 2), 16);
    var b = parseInt(hexcolor.substr(4, 2), 16);
    var yiq = ((r * 299) + (g * 587) + (b * 114)) / 1000;
    return (yiq >= 128) ? 'black' : 'white';
}
function getColor(colors, color, shade) {
    if (colors[color]) {
        var shadeColor = colors[color].color;
        if (typeof shadeColor === 'string') {
            return colors[color].color;
        }
        else {
            return colors[color].color[shade];
        }
    }
    else {
        if (typeof color === 'string') {
            return color;
        }
        else {
            return color[shade];
        }
    }
}
var BgAndColorStyle = /** @class */ (function () {
    function BgAndColorStyle() {
    }
    return BgAndColorStyle;
}());
var LyStyleTheme = /** @class */ (function () {
    function LyStyleTheme(theme) {
        this.theme = theme;
    }
    LyStyleTheme.decorators = [
        { type: core.Injectable },
    ];
    /**
       * for bg or color inputs, find the color in palette, return color
       * example: this.theme.update('bg', 'primary', this.theme.AlyleUI.palette); // #00bcd4
       */
    // getPalette(color: string, colors?: any): string {
    //   let result: string;
    //   if (colors) {
    //     result = getColor(colors, color);
    //   } else {
    //     result = getColor(this.theme.AlyleUI.palette, color);
    //   }
    //   return result;
    // }
    /** @nocollapse */
    LyStyleTheme.ctorParameters = function () { return [
        { type: LyTheme, },
    ]; };
    return LyStyleTheme;
}());
/**
 * -------------
 */
var LyPalette = /** @class */ (function () {
    function LyPalette() {
        this._primary = {
            name: 'none',
            color: '#fff',
            text: '#fff',
        };
        this._accent = {
            name: 'none',
            color: '#fff',
            text: '#fff',
        };
        this._other = {
            name: 'none',
            color: '#fff',
            text: '#fff',
        };
        this.themeApp = {
            light: {
                color: '#24292e',
                background: '#f5f5f5',
            },
            dark: {
                color: '#f5f5f5',
                background: '#2C2D31',
            },
        };
        this.text = {
            default: 'rgba(0, 0, 0, 0.75)',
            light: '#fff',
            dark: '#2C2D31',
        };
    }
    LyPalette.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    LyPalette.ctorParameters = function () { return []; };
    return LyPalette;
}());

var chroma = _chroma;
var shadowKeyUmbraOpacity = 0.2;
var shadowKeyPenumbraOpacity = 0.14;
var shadowAmbientShadowOpacity = 0.12;
var elevations = [
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
    [0, 1, 3, 0, 0, 1, 1, 0, 0, 2, 1, -1],
    [0, 1, 5, 0, 0, 2, 2, 0, 0, 3, 1, -2],
    [0, 1, 8, 0, 0, 3, 4, 0, 0, 3, 3, -2],
    [0, 2, 4, -1, 0, 4, 5, 0, 0, 1, 10, 0],
    [0, 3, 5, -1, 0, 5, 8, 0, 0, 1, 14, 0],
    [0, 3, 5, -1, 0, 6, 10, 0, 0, 1, 18, 0],
    [0, 4, 5, -2, 0, 7, 10, 1, 0, 2, 16, 1],
    [0, 5, 5, -3, 0, 8, 10, 1, 0, 3, 14, 2],
    [0, 5, 6, -3, 0, 9, 12, 1, 0, 3, 16, 2],
    [0, 6, 6, -3, 0, 10, 14, 1, 0, 4, 18, 3],
    [0, 6, 7, -4, 0, 11, 15, 1, 0, 4, 20, 3],
    [0, 7, 8, -4, 0, 12, 17, 2, 0, 5, 22, 4],
    [0, 7, 8, -4, 0, 13, 19, 2, 0, 5, 24, 4],
    [0, 7, 9, -4, 0, 14, 21, 2, 0, 5, 26, 4],
    [0, 8, 9, -5, 0, 15, 22, 2, 0, 6, 28, 5],
    [0, 8, 10, -5, 0, 16, 24, 2, 0, 6, 30, 5],
    [0, 8, 11, -5, 0, 17, 26, 2, 0, 6, 32, 5],
    [0, 9, 11, -5, 0, 18, 28, 2, 0, 7, 34, 6],
    [0, 9, 12, -6, 0, 19, 29, 2, 0, 7, 36, 6],
    [0, 10, 13, -6, 0, 20, 31, 3, 0, 8, 38, 7],
    [0, 10, 13, -6, 0, 21, 33, 3, 0, 8, 40, 7],
    [0, 10, 14, -6, 0, 22, 35, 3, 0, 8, 42, 7],
    [0, 11, 14, -7, 0, 23, 36, 3, 0, 9, 44, 8],
    [0, 11, 15, -7, 0, 24, 38, 3, 0, 9, 46, 8]
];
function shadowBuilder(elevation, color) {
    var Color = chroma(color || '#000');
    var colors = [
        Color.alpha(shadowKeyUmbraOpacity).css(),
        Color.alpha(shadowKeyPenumbraOpacity).css(),
        Color.alpha(shadowAmbientShadowOpacity).css()
    ];
    var e = elevations[elevation];
    // tslint:disable-next-line:max-line-length
    return "box-shadow:" + e[0] + "px " + e[1] + "px " + e[2] + "px " + e[3] + "px " + colors[0] + "," + e[4] + "px " + e[5] + "px " + e[6] + "px " + e[7] + "px " + colors[1] + "," + e[8] + "px " + e[9] + "px " + e[10] + "px " + e[11] + "px " + colors[2];
}

var RandomId = /** @class */ (function () {
    function RandomId() {
    }
    Object.defineProperty(RandomId.prototype, "generate", {
        get: function () {
            var id = (Math.random() + Date.now());
            return id;
        },
        enumerable: true,
        configurable: true
    });
    RandomId.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    RandomId.ctorParameters = function () { return []; };
    return RandomId;
}());

function Ly_Next(value, index) {
    var nxt = value;
    var i = index;
    var length = nxt.length;
    var _index;
    // let part = index / length;
    if ((i + 1) < length) {
        nxt = (nxt[(i + 1)]);
        _index = i + 1;
    }
    else {
        _index = 0;
        nxt = (nxt[(0)]);
    }
    return {
        index: _index,
        next: nxt,
    };
}

function Ly_Prev(value, index) {
    var nxt = value;
    var i = index;
    var length = nxt.length;
    var _index;
    if ((i - 1) <= 0 || (i - 1) >= length - 1) {
        nxt = (nxt[(nxt.length - 1)]);
        _index = (nxt.length - 1);
    }
    else if (i === 0) {
        nxt = (nxt[(0)]);
        _index = (0);
    }
    else {
        nxt = (nxt[(i - 1)]);
        _index = (i - 1);
    }
    return {
        index: _index,
        prev: nxt,
    };
}

function getParents(el, parentSelector) {
    // If no parentSelector defined will bubble up all the way to *document*
    if (parentSelector === undefined) {
        parentSelector = 'body';
    }
    var parents = [];
    var p = el.parentNode;
    var pxz = null;
    while (!pxz) {
        var o = p;
        parents.push(o);
        p = o.parentNode;
        pxz = p.querySelector(parentSelector);
    }
    // parents.push(_parentSelector); // Push that parentSelector you wanted to stop at
    // console.log(parents[parents.length - 1]);
    return parents[parents.length - 1];
}

// Whether the current platform supports the V8 Break Iterator. The V8 check
// is necessary to detect all Blink based browsers.
var hasV8BreakIterator = (typeof (Intl) !== 'undefined' && Intl.v8BreakIterator);
/**
 * Service to detect the current platform by comparing the userAgent strings and
 * checking browser-specific global properties.
 */
var Platform = /** @class */ (function () {
    function Platform() {
        /** Layout Engines */
        this.EDGE = Platform.isBrowser && /(edge)/i.test(navigator.userAgent);
        this.TRIDENT = Platform.isBrowser && /(msie|trident)/i.test(navigator.userAgent);
        // EdgeHTML and Trident mock Blink specific things and need to be excluded from this check.
        this.BLINK = Platform.isBrowser &&
            (!!(window.chrome || hasV8BreakIterator) && !!CSS && !this.EDGE && !this.TRIDENT);
        // Webkit is part of the userAgent in EdgeHTML, Blink and Trident. Therefore we need to
        // ensure that Webkit runs standalone and is not used as another engine's base.
        this.WEBKIT = Platform.isBrowser &&
            /AppleWebKit/i.test(navigator.userAgent) && !this.BLINK && !this.EDGE && !this.TRIDENT;
        /** Browsers and Platform Types */
        this.IOS = Platform.isBrowser && /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;
        // It's difficult to detect the plain Gecko engine, because most of the browsers identify
        // them self as Gecko-like browsers and modify the userAgent's according to that.
        // Since we only cover one explicit Firefox case, we can simply check for Firefox
        // instead of having an unstable check for Gecko.
        this.FIREFOX = Platform.isBrowser && /(firefox|minefield)/i.test(navigator.userAgent);
        // Trident on mobile adds the android platform to the userAgent to trick detections.
        this.ANDROID = Platform.isBrowser && /android/i.test(navigator.userAgent) && !this.TRIDENT;
        // Safari browsers will include the Safari keyword in their userAgent. Some browsers may fake
        // this and just place the Safari keyword in the userAgent. To be more safe about Safari every
        // Safari browser should also use Webkit as its layout engine.
        this.SAFARI = Platform.isBrowser && /safari/i.test(navigator.userAgent) && this.WEBKIT;
    }
    Platform.isBrowser = typeof document === 'object' && !!document;
    return Platform;
}());

var LyRoot = /** @class */ (function () {
    function LyRoot(elementRef, theme) {
        var _this = this;
        this.elementRef = elementRef;
        this.theme = theme;
        this.themeSubscription = this.theme.palette.subscribe(function (palette) {
            _this.backgroundStyle = palette['bgText'];
            _this.colorStyle = palette['colorText'];
            _this.fontFamily = theme.AlyleUI.currentTheme.typography.fontFamily;
        });
    }
    LyRoot.prototype.ngOnDestroy = function () {
        this.themeSubscription.unsubscribe();
    };
    LyRoot.decorators = [
        { type: core.Component, args: [{
                    selector: 'ly-core[root]',
                    template: "\n  <ng-content></ng-content>\n  ",
                    styles: ["html, body { width: 100%; height: 100%; } body { margin: 0; } ly-core[root] { display: block; position: relative; } ly-svg svg, [ly-svg] svg { width: inherit; height: inherit; fill: currentColor; display: block; } @keyframes btnFocusAnim { from { transform: scale(0.6); } 50% { transform: scale(0.8); } to { transform: scale(0.6); } } [ly-ripple] > ly-hover-container { position: absolute; top: 0; left: 0; background: currentColor; opacity: 0; transform: scale3d(0, 0, 0); transition: all 0ms cubic-bezier(0.45, 0, 0.25, 1); border-radius: 100%; } button { -webkit-tap-highlight-color: transparent; } [ly-icon-button] > .ly-icon-button-container > .ly-icon-button-content > ly-hover-container { background: -moz-radial-gradient(center, ellipse cover, currentColor 0%, transparent 100%); /* FF3.6-15 */ background: -webkit-radial-gradient(center, ellipse cover, currentColor 0%, transparent 100%); /* Chrome10-25,Safari5.1-6 */ background: radial-gradient(ellipse at center, currentColor 0%, transparent 100%); /* W3C, IE10+, FF16+, Chrome26+, Opera12+, Safari7+ */ } [ly-ripple]:not(.ly-ripple-no-focus):focus > ly-hover-container, [ly-button]:not(.ly-ripple-no-focus):focus > .ly-button-ripple > ly-hover-container { animation: btnFocusAnim infinite 1.4s 0ms; transform: scale3d(1, 1, 1); opacity: .13; } [ly-icon-button]:not(.ly-ripple-no-focus):focus > .ly-icon-button-container > [ly-ripple] > ly-hover-container { /* for ly-icon-button*/ transform: scale3d(1, 1, 1); opacity: .33; transition: all 375ms cubic-bezier(0.45, 0, 0.25, 1); } /* LyRippleMinimal */ .ly-ripple-minimal { position: relative; } .ly-ripple-minimal .ly-ripple-container { position: absolute; width: 5px; height: 5px; background: currentColor; opacity: .16; border-radius: 100%; transform: scale(0); transition: opacity ease, transform cubic-bezier(0.4, 0, 0.2, 1); pointer-events: none; } ly-drawer-container { display: flex; } ly-drawer-container .ly-drawer { position: fixed; overflow: auto; height: inherit; width: inherit; top: inherit; bottom: inherit; text-align: inherit; } ly-drawer-container ly-drawer-content { flex: 1 1; } "],
                    encapsulation: core.ViewEncapsulation.None,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false
                },] },
    ];
    /** @nocollapse */
    LyRoot.ctorParameters = function () { return [
        { type: core.ElementRef, },
        { type: LyTheme, },
    ]; };
    LyRoot.propDecorators = {
        "backgroundStyle": [{ type: core.HostBinding, args: ['style.background',] },],
        "colorStyle": [{ type: core.HostBinding, args: ['style.color',] },],
        "fontFamily": [{ type: core.HostBinding, args: ['style.font-family',] },],
    };
    return LyRoot;
}());
var LyInlineText = /** @class */ (function () {
    function LyInlineText() {
    }
    LyInlineText.decorators = [
        { type: core.Component, args: [{
                    selector: 'inline-text',
                    template: "<ng-content></ng-content>",
                    styles: ["\n    :host {\n      vertical-align: baseline;\n    }\n  "],
                },] },
    ];
    /** @nocollapse */
    LyInlineText.ctorParameters = function () { return []; };
    return LyInlineText;
}());
var LyCoreModule = /** @class */ (function () {
    function LyCoreModule() {
    }
    LyCoreModule.forRoot = function () {
        return {
            ngModule: LyCoreModule,
        };
    };
    LyCoreModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule],
                    exports: [LyRoot, LyInlineText],
                    declarations: [LyRoot, LyInlineText],
                    providers: [Platform]
                },] },
    ];
    /** @nocollapse */
    LyCoreModule.ctorParameters = function () { return []; };
    return LyCoreModule;
}());

/**
 * Color tranformation
 */
function getHexValue(intInput) {
    var result = intInput.toString(16);
    if (result.length < 2) {
        result = '0' + result;
    }
    return result;
}
function Color(red, green, blue, alpha) {
    return {
        red: red,
        green: green,
        blue: blue,
        alpha: alpha,
        toHex: function () {
            return getHexValue(red) + getHexValue(green) + getHexValue(blue);
        }
    };
}
var ɵ0 = function (color, bgColor) {
    var alpha = color.alpha;
    function getTintValue(tint, bgTint) {
        var tmp = Math.floor((1 - alpha) * bgTint + alpha * tint);
        if (tmp > 255) {
            return 255;
        }
        return tmp;
    }
    return Color(getTintValue(color.red, bgColor.red), getTintValue(color.green, bgColor.green), getTintValue(color.blue, bgColor.blue));
};
// Converter which actually does the calculation from rgba to hex.
var colorConverter = {
    // Converts the given color to a Color object, using the given gbColor in the calculation.
    convertToHex: ɵ0
};
function hue2rgb(p, q, t) {
    if (t < 0) {
        t += 1;
    }
    if (t > 1) {
        t -= 1;
    }
    if (t < 1 / 6) {
        return p + (q - p) * 6 * t;
    }
    if (t < 1 / 2) {
        return q;
    }
    if (t < 2 / 3) {
        return p + (q - p) * (2 / 3 - t) * 6;
    }
    return p;
}
var ɵ1 = function (rgba) {
    // Strip the rgba-definition off the string.
    rgba = rgba.replace('rgba(', '')
        .replace(')', '')
        .replace(' ', '');
    // Split the rgba string into an array.
    var splittedRgba = rgba.split(',');
    return Color(parseInt(splittedRgba[0], 10), parseInt(splittedRgba[1], 10), parseInt(splittedRgba[2], 10), parseFloat(splittedRgba[3]) || 1);
}, ɵ2 = function (hexString) {
    hexString = hexString.replace('#', '');
    var rgbArr = [];
    function getHexPartByIndex(index) {
        switch (hexString.length) {
            case 3:
                return hexString[index] + hexString[index];
            default:
                index *= 2;
                return hexString[index] + hexString[index + 1];
        }
    }
    // String the "#" off the hex-string.
    hexString = hexString.replace('#', '');
    // Convert pairs of hex-characters into decimal numbers.
    for (var i = 0; i < hexString.length; i++) {
        rgbArr.push(parseInt(getHexPartByIndex(i), 16));
    }
    return Color(rgbArr[0], rgbArr[1], rgbArr[2], 1);
}, ɵ3 = function hslToRgb(hsla) {
    hsla = hsla.replace('hsla(', '')
        .replace('hsl(', '')
        .replace(')', '')
        .replace(' ', '');
    // Split the hsla string into an array.
    hsla = hsla.split(',');
    var h = parseInt(hsla[0], 10) / 360;
    var s = parseInt(hsla[1], 10) / 100;
    var l = parseInt(hsla[2], 10) / 100;
    var a = parseFloat(hsla[3]) || 1;
    var r, g, b;
    if (s == 0) {
        r = g = b = l; // achromatic
    }
    else {
        var q = l < 0.5 ? l * (1 + s) : l + s - l * s;
        var p = 2 * l - q;
        r = hue2rgb(p, q, h + 1 / 3);
        g = hue2rgb(p, q, h);
        b = hue2rgb(p, q, h - 1 / 3);
    }
    return Color(Math.round(r * 255), Math.round(g * 255), Math.round(b * 255), a);
};
var colorStringParser = {
    // Converter for rgb(a) colors
    rgba: ɵ1,
    // Converter for hex colors
    hex: ɵ2,
    // Converter for hsl(a) colors.
    hsla: ɵ3
};
// Returns the type of color based on the given string.
function getColorType(colorString) {
    if (colorString.indexOf('rgba') !== -1 || colorString.indexOf('rgb') !== -1) {
        return 'rgba';
    }
    if (colorString.indexOf('hsla') !== -1 || colorString.indexOf('hsl') !== -1) {
        return 'hsla';
    }
    return 'hex';
}
// Convenience function that returns the Color object for the given inputString.
function getColorForString(inputString) {
    return colorStringParser[getColorType(inputString)](inputString);
}
function trim(str) {
    return str.replace(/^\s+|\s+$/gm, '');
}
function rgbaToHex(rgba) {
    var parts = rgba.substring(rgba.indexOf('(')).split(','), r = parseInt(trim(parts[0].substring(1)), 10), g = parseInt(trim(parts[1]), 10), b = parseInt(trim(parts[2]), 10), a = parseFloat(trim(parts[3].substring(0, parts[3].length - 1))).toFixed(2);
    return ('#' + r.toString(16) + g.toString(16) + b.toString(16) + (Number(a) * 255).toString(16).substring(0, 2));
}
function componentToHex(c) {
    c = Number(c);
    var hex = c.toString(16);
    return hex.length === 1 ? '0' + hex : hex;
}
// for shadow
function toRgb(str, alpha) {
    return hexToRgba(colorConverter.convertToHex(getColorForString(str), getColorForString('#ffffff')).toHex(), alpha || 1);
}
function converterToHex(str) {
    return colorConverter.convertToHex(getColorForString(str), getColorForString('#ffffff')).toHex();
}
function rgbToHex(rgb) {
    var toRgb = rgb;
    if (toRgb.substr(0, 4) === 'rgba') {
        toRgb = toRgb.slice(5);
    }
    else {
        toRgb = toRgb.slice(4);
    }
    toRgb = toRgb.slice(0, -1);
    toRgb = toRgb.replace(' ', '');
    toRgb = toRgb.split(',');
    return componentToHex(toRgb[0]) + componentToHex(toRgb[1]) + componentToHex(toRgb[2]);
}
function rgbTransparent(rgba) {
    if (typeof rgba === 'string') {
        rgba = rgba.slice(0, -1);
        rgba = (rgba.split(',')[3]) * 1;
        if (typeof rgba === 'number') {
            rgba = 1;
        }
    }
    else {
        rgba = 1;
    }
    return rgba;
}
function hexToRgba(hex, tr) {
    if (tr === void 0) { tr = -1; }
    var _default = hex;
    var transparent = {
        r: 0,
        g: 0,
        b: 0,
        a: 0
    };
    if (tr === -1) {
        tr = rgbTransparent(hex);
    }
    hex = String(hex);
    if ((hex.substr(0, 4) === 'rgba') && !!hex.split(',')[3]) {
        tr = parseFloat(hex.split(',')[3].replace(')', ''));
        hex = String(hex);
        if (hex.length !== 6 && hex.length !== 3) {
            hex = rgbToHex(hex);
        }
    }
    if (hex !== undefined && hex !== null && hex !== 'transparent') {
        // Expand shorthand form (e.g. "03F") to full form (e.g. "0033FF")
        var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
        hex = hex.replace(shorthandRegex, function (m, r, g, b) {
            return r + r + g + g + b + b;
        });
        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
        var colorr = 0;
        var colorg = 0;
        var colorb = 0;
        if (result) {
            colorr = parseInt(result[1], 16);
            colorg = parseInt(result[2], 16);
            colorb = parseInt(result[3], 16);
        }
        return result ? {
            json: {
                r: colorr,
                g: colorg,
                b: colorb,
                a: tr
            },
            color: "rgba(" + colorr + ", " + colorg + ", " + colorb + ", " + tr + ")",
        } : {
            json: transparent,
            color: "" + _default,
        };
    }
    else {
        return {
            json: transparent,
            color: "rgba(" + 0 + ", " + 0 + ", " + 0 + ", " + 0 + ")",
        };
    }
}

var LyBg = /** @class */ (function () {
    function LyBg(theme, renderer, elementRef) {
        this.theme = theme;
        this.renderer = renderer;
        this.elementRef = elementRef;
        /** Default bg */
        this._bg = 'primary';
        this.prefix = 'bg';
    }
    Object.defineProperty(LyBg.prototype, "bg", {
        get: function () {
            return this._bg;
        },
        set: function (color) {
            this._bg = color;
            var key = "" + this.prefix + (color || this._bg);
            var newStyle = this.theme.createStyle("ly-" + key, this.css.bind(this), color);
            this.theme.updateClass(this.elementRef, this.renderer, newStyle.id, this._lastClass);
            this._lastClass = newStyle.id;
        },
        enumerable: true,
        configurable: true
    });
    LyBg.prototype.css = function (bg) {
        return "background:" + this.theme.colorOf(bg) + ";";
    };
    LyBg.decorators = [
        { type: core.Directive, args: [{
                    selector: '[bg]'
                },] },
    ];
    /** @nocollapse */
    LyBg.ctorParameters = function () { return [
        { type: LyTheme, },
        { type: core.Renderer2, },
        { type: core.ElementRef, },
    ]; };
    LyBg.propDecorators = {
        "bg": [{ type: core.Input, args: ['bg',] },],
    };
    return LyBg;
}());

var BgModule = /** @class */ (function () {
    function BgModule() {
    }
    BgModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    exports: [LyBg],
                    declarations: [LyBg]
                },] },
    ];
    /** @nocollapse */
    BgModule.ctorParameters = function () { return []; };
    return BgModule;
}());

var LyColor = /** @class */ (function () {
    function LyColor(theme, renderer, elementRef) {
        this.theme = theme;
        this.renderer = renderer;
        this.elementRef = elementRef;
        /** Default color */
        this._color = 'primary';
        this.prefix = 'color';
    }
    Object.defineProperty(LyColor.prototype, "color", {
        get: function () {
            return this._color;
        },
        set: function (color) {
            this._color = color;
            var key = "" + this.prefix + (color || this._color);
            var newStyle = this.theme.createStyle("ly-" + key, this.css.bind(this), color);
            this.theme.updateClass(this.elementRef, this.renderer, newStyle.id, this._lastClass);
            this._lastClass = newStyle.id;
        },
        enumerable: true,
        configurable: true
    });
    LyColor.prototype.css = function (color) {
        return "color:" + this.theme.colorOf(color) + ";";
    };
    LyColor.decorators = [
        { type: core.Directive, args: [{
                    selector: '[color]'
                },] },
    ];
    /** @nocollapse */
    LyColor.ctorParameters = function () { return [
        { type: LyTheme, },
        { type: core.Renderer2, },
        { type: core.ElementRef, },
    ]; };
    LyColor.propDecorators = {
        "color": [{ type: core.Input, args: ['color',] },],
    };
    return LyColor;
}());

var ColorModule = /** @class */ (function () {
    function ColorModule() {
    }
    ColorModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    exports: [LyColor],
                    declarations: [LyColor]
                },] },
    ];
    /** @nocollapse */
    ColorModule.ctorParameters = function () { return []; };
    return ColorModule;
}());

var ThemeModule = /** @class */ (function () {
    function ThemeModule() {
    }
    ThemeModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        BgModule,
                        ColorModule
                    ],
                    exports: [BgModule, ColorModule]
                },] },
    ];
    /** @nocollapse */
    ThemeModule.ctorParameters = function () { return []; };
    return ThemeModule;
}());

var NgTranscludeDirective = /** @class */ (function () {
    function NgTranscludeDirective(_viewRef) {
        this._viewRef = _viewRef;
        this.ngTranscludeChange = new core.EventEmitter();
        this.viewRef = _viewRef;
    }
    Object.defineProperty(NgTranscludeDirective.prototype, "ngTransclude", {
        get: function () {
            return this._ngTransclude;
        },
        set: function (templateRef) {
            this._ngTransclude = templateRef;
            if (templateRef) {
                this.viewRef.createEmbeddedView(templateRef);
            }
        },
        enumerable: true,
        configurable: true
    });
    NgTranscludeDirective.prototype.ngAfterContentInit = function () {
        this.ngTranscludeChange.emit(true);
    };
    NgTranscludeDirective.prototype.ngOnDestroy = function () {
        this.viewRef.detach();
    };
    NgTranscludeDirective.decorators = [
        { type: core.Directive, args: [{
                    selector: '[ngTransclude]'
                },] },
    ];
    /** @nocollapse */
    NgTranscludeDirective.ctorParameters = function () { return [
        { type: core.ViewContainerRef, },
    ]; };
    NgTranscludeDirective.propDecorators = {
        "ngTransclude": [{ type: core.Input },],
        "ngTranscludeChange": [{ type: core.Output },],
    };
    return NgTranscludeDirective;
}());
var NgTranscludeModule = /** @class */ (function () {
    function NgTranscludeModule() {
    }
    NgTranscludeModule.decorators = [
        { type: core.NgModule, args: [{
                    exports: [NgTranscludeDirective],
                    declarations: [NgTranscludeDirective]
                },] },
    ];
    /** @nocollapse */
    NgTranscludeModule.ctorParameters = function () { return []; };
    return NgTranscludeModule;
}());

function isWindow(obj) {
    return obj !== null && obj === obj.window;
}
function getWindow(elem) {
    return isWindow(elem) ? elem : elem.nodeType === 9 && elem.defaultView;
}
function exactPosition(elem) {
    var docElem, win, box = { top: 0, left: 0 };
    var doc = elem && elem.ownerDocument;
    docElem = doc.documentElement;
    if (typeof elem.getBoundingClientRect !== typeof undefined) {
        box = elem.getBoundingClientRect();
    }
    win = getWindow(doc);
    return {
        top: box.top + win.pageYOffset - docElem.clientTop,
        left: box.left + win.pageXOffset - docElem.clientLeft
    };
}

function toBoolean(value) {
    return value != null && "" + value !== 'false';
}
function IsBoolean() {
    return function (target, key) {
        var definition = Object.getOwnPropertyDescriptor(target, key);
        if (definition) {
            Object.defineProperty(target, key, {
                get: definition.get,
                set: function (newValue) {
                    definition.set(toBoolean(newValue));
                },
                enumerable: true,
                configurable: true
            });
        }
        else {
            Object.defineProperty(target, key, {
                get: function () {
                    return this['__' + key];
                },
                set: function (newValue) {
                    this['__' + key] = toBoolean(newValue);
                },
                enumerable: true,
                configurable: true
            });
        }
    };
}

var LyOverlayContainer = /** @class */ (function () {
    function LyOverlayContainer() {
        if (Platform.isBrowser) {
            var container = document.createElement('ly-overlay-container');
            document.body.appendChild(container);
            this._containerElement = container;
        }
    }
    Object.defineProperty(LyOverlayContainer.prototype, "containerElement", {
        get: function () {
            return this._containerElement;
        },
        enumerable: true,
        configurable: true
    });
    LyOverlayContainer.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    LyOverlayContainer.ctorParameters = function () { return []; };
    return LyOverlayContainer;
}());

var DomService = /** @class */ (function () {
    function DomService(componentFactoryResolver, overlayContainer) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.overlayContainer = overlayContainer;
    }
    DomService.prototype.attach = function (_hostViewContainerRef, component, template) {
        var _this = this;
        var viewRef = _hostViewContainerRef.createEmbeddedView(template);
        viewRef.detectChanges();
        this._viewContainerRef = _hostViewContainerRef;
        viewRef.rootNodes.forEach(function (rootNode) { return _this.addChild(rootNode); });
    };
    DomService.prototype.addChild = function (child) {
        this.overlayContainer.containerElement.appendChild(child);
    };
    DomService.prototype.getDomElementFromComponentRef = function (componentRef) {
        return componentRef.hostView
            .rootNodes[0];
    };
    DomService.prototype.destroyRef = function (componentRef, delay) {
        var _this = this;
        setTimeout(function () {
            if (_this._viewContainerRef) {
                _this._viewContainerRef.detach();
                _this._viewContainerRef.clear();
            }
        }, delay);
    };
    DomService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    DomService.ctorParameters = function () { return [
        { type: core.ComponentFactoryResolver, },
        { type: LyOverlayContainer, },
    ]; };
    return DomService;
}());

function LY_OVERLAY_CONTAINER_PROVIDER_FACTORY(parentContainer) {
    return parentContainer || new LyOverlayContainer();
}
var LY_OVERLAY_CONTAINER_PROVIDER = {
    // If there is already an OverlayContainer available, use that. Otherwise, provide a new one.
    provide: LyOverlayContainer,
    deps: [[new core.Optional(), new core.SkipSelf(), LyOverlayContainer]],
    useFactory: LY_OVERLAY_CONTAINER_PROVIDER_FACTORY
};
var LxDomModule = /** @class */ (function () {
    function LxDomModule() {
    }
    LxDomModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    providers: [[DomService, LY_OVERLAY_CONTAINER_PROVIDER]]
                },] },
    ];
    /** @nocollapse */
    LxDomModule.ctorParameters = function () { return []; };
    return LxDomModule;
}());

(function (FocusStatus) {
    /**mouse and/or touch*/
    FocusStatus["DEFAULT"] = "default";
    /** keyboard and/or program*/
    FocusStatus["KEYBOARD"] = "keyboard";
})(exports.FocusStatus || (exports.FocusStatus = {}));
var LyFocusState = /** @class */ (function () {
    function LyFocusState(elementRef, _ngZone, _renderer, _cd) {
        var _this = this;
        this._ngZone = _ngZone;
        this._renderer = _renderer;
        this.stateMap = new Map();
        this._eventHandlers = new Map();
        this._stateSubject = new Subject.Subject();
        this.lyFocusChange = new core.EventEmitter();
        if (Platform.isBrowser) {
            this._eventHandlers
                .set('focus', this.on.bind(this))
                .set('blur', this.on.bind(this))
                .set('touchstart', this.on.bind(this))
                .set('mousedown', this.on.bind(this));
            var element = elementRef.nativeElement;
            this.setTriggerElement(element);
            var ob = this._stateSubject.asObservable();
            this._stateSubscription = ob
                .debounceTime(111)
                .subscribe(function (e) {
                _this.state = e;
                _this._updateClass();
                _this.lyFocusChange.emit(e);
            });
        }
    }
    LyFocusState.prototype._updateState = function () {
        var state;
        if (this.stateMap.has('blur')) {
            this.stateMap.clear();
        }
        else if (this.stateMap.has('focus') && this.stateMap.has('mousedown') || this.stateMap.has('focus') && this.stateMap.has('touchstart')) {
            state = exports.FocusStatus.DEFAULT;
        }
        else if (this.stateMap.has('focus')) {
            state = exports.FocusStatus.KEYBOARD;
        }
        this._stateSubject.next(state);
    };
    LyFocusState.prototype.on = function (event) {
        this.stateMap.set(event.type, true);
        this._updateState();
    };
    LyFocusState.prototype._updateClass = function () {
        var _this = this;
        var element = this._containerElement;
        var state = this.state;
        var toggleClass = function (className, shouldSet) { return shouldSet ? _this._renderer.addClass(element, className) : _this._renderer.removeClass(element, className); };
        toggleClass("ly-focused", !!state);
        for (var key in exports.FocusStatus) {
            if (exports.FocusStatus.hasOwnProperty(key)) {
                var className = exports.FocusStatus[key];
                toggleClass("ly-" + className + "-focused", state === className);
            }
        }
    };
    LyFocusState.prototype.setTriggerElement = function (element) {
        var _this = this;
        if (this._containerElement) {
            this._eventHandlers.forEach(function (fn, type) {
                _this._containerElement.removeEventListener(type, fn);
            });
        }
        if (element) {
            this._ngZone.runOutsideAngular(function () {
                return _this._eventHandlers.forEach(function (fn, type) {
                    return element.addEventListener(type, fn);
                });
            });
        }
        this._containerElement = element;
    };
    LyFocusState.prototype.ngOnDestroy = function () {
        if (Platform.isBrowser) {
            this._stateSubscription.unsubscribe();
            this.setTriggerElement(null);
        }
    };
    LyFocusState.decorators = [
        { type: core.Directive, args: [{
                    selector: '[lyFocusState]',
                    exportAs: 'lyFocusState'
                },] },
    ];
    /** @nocollapse */
    LyFocusState.ctorParameters = function () { return [
        { type: core.ElementRef, },
        { type: core.NgZone, },
        { type: core.Renderer2, },
        { type: core.ChangeDetectorRef, },
    ]; };
    LyFocusState.propDecorators = {
        "lyFocusChange": [{ type: core.Output },],
    };
    return LyFocusState;
}());

var LyFocusStateModule = /** @class */ (function () {
    function LyFocusStateModule() {
    }
    LyFocusStateModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [LyFocusState],
                    exports: [LyFocusState]
                },] },
    ];
    /** @nocollapse */
    LyFocusStateModule.ctorParameters = function () { return []; };
    return LyFocusStateModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

exports.ɵa = LyOverlayContainer;
exports.getContrastYIQ = getContrastYIQ;
exports.getColor = getColor;
exports.BgAndColorStyle = BgAndColorStyle;
exports.LyStyleTheme = LyStyleTheme;
exports.LyPalette = LyPalette;
exports.ThemeColor = ThemeColor;
exports.themeProperty = themeProperty;
exports.LyTheme = LyTheme;
exports.isObject = isObject;
exports.mergeDeep = mergeDeep;
exports.parsePalette = parsePalette;
exports.shadowBuilder = shadowBuilder;
exports.RandomId = RandomId;
exports.Ly_Next = Ly_Next;
exports.Ly_Prev = Ly_Prev;
exports.getParents = getParents;
exports.AlyleServiceConfig = AlyleServiceConfig;
exports.LyRoot = LyRoot;
exports.LyInlineText = LyInlineText;
exports.LyCoreModule = LyCoreModule;
exports.Platform = Platform;
exports.Color = Color;
exports.trim = trim;
exports.rgbaToHex = rgbaToHex;
exports.componentToHex = componentToHex;
exports.toRgb = toRgb;
exports.converterToHex = converterToHex;
exports.rgbToHex = rgbToHex;
exports.rgbTransparent = rgbTransparent;
exports.hexToRgba = hexToRgba;
exports.ɵ0 = ɵ0;
exports.ɵ1 = ɵ1;
exports.ɵ2 = ɵ2;
exports.ɵ3 = ɵ3;
exports.ThemeModule = ThemeModule;
exports.BgModule = BgModule;
exports.ColorModule = ColorModule;
exports.LyBg = LyBg;
exports.LyColor = LyColor;
exports.NgTranscludeDirective = NgTranscludeDirective;
exports.NgTranscludeModule = NgTranscludeModule;
exports.exactPosition = exactPosition;
exports.toBoolean = toBoolean;
exports.IsBoolean = IsBoolean;
exports.gradStop = gradStop;
exports.computeStops = computeStops;
exports.DomService = DomService;
exports.LY_OVERLAY_CONTAINER_PROVIDER_FACTORY = LY_OVERLAY_CONTAINER_PROVIDER_FACTORY;
exports.LY_OVERLAY_CONTAINER_PROVIDER = LY_OVERLAY_CONTAINER_PROVIDER;
exports.LxDomModule = LxDomModule;
exports.LyFocusStateModule = LyFocusStateModule;
exports.LyFocusState = LyFocusState;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=alyle-ui__core.umd.js.map
