(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('alyle-ui/core'), require('@angular/core'), require('@angular/common')) :
	typeof define === 'function' && define.amd ? define(['exports', 'alyle-ui/core', '@angular/core', '@angular/common'], factory) :
	(factory((global.alyleUi = global.alyleUi || {}, global.alyleUi.rippleMinimal = {}),global.ly.core,global.ng.core,global.ng.common));
}(this, (function (exports,core,core$1,common) { 'use strict';

var RippleRef = /** @class */ (function () {
    function RippleRef() {
        this.state = true;
        this.timestamp = -Date.now();
        this.container = document.createElement('span');
    }
    RippleRef.prototype.end = function () {
        this.state = false;
        this.timestamp += Date.now();
    };
    return RippleRef;
}());
var Ripple = /** @class */ (function () {
    function Ripple(_elementRef, _renderer, _ngZone) {
        this._renderer = _renderer;
        this._ngZone = _ngZone;
        this._state = true;
        this._eventHandlers = new Map();
        this.rippleConfig = {};
        this._transitionDuration = '475ms';
        if (core.Platform.isBrowser) {
            if (typeof TouchEvent && !!TouchEvent) {
                this._eventHandlers.set('pointerdown', this.onPointerDown.bind(this));
                this._eventHandlers.set('touchend', this.onPointerLeave.bind(this));
            }
            else {
                this._eventHandlers.set('mousedown', this.onPointerDown.bind(this));
            }
            this._eventHandlers.set('mouseup', this.onPointerLeave.bind(this));
            this._eventHandlers.set('mouseleave', this.onPointerLeave.bind(this));
            this._hostElement = _elementRef;
            this.setTriggerElement(this._hostElement);
        }
    }
    Object.defineProperty(Ripple.prototype, "_rectContainer", {
        get: function () {
            return this._hostElement.getBoundingClientRect();
        },
        enumerable: true,
        configurable: true
    });
    Ripple.prototype.setContainerElement = function (element) {
        this._hostElement = element;
    };
    Ripple.prototype.setTriggerElement = function (element) {
        var _this = this;
        if (this._triggerElement) {
            // this._renderer.removeClass(this._triggerElement, 'ly-ripple-minimal');
            this._eventHandlers.forEach(function (fn, type) {
                _this._triggerElement.removeEventListener(type, fn);
            });
        }
        if (element) {
            this._renderer.addClass(element, 'ly-ripple-minimal');
            this._ngZone.runOutsideAngular(function () {
                _this._eventHandlers.forEach(function (fn, type) { return element.addEventListener(type, fn); });
            });
        }
        this._triggerElement = element;
    };
    Ripple.prototype.createRipple = function (styles) {
        this._rippleRef = new RippleRef();
        var container = this._rippleRef.container;
        container.className = "ly-ripple-container";
        for (var key in styles) {
            if (styles.hasOwnProperty(key)) {
                var element = styles[key];
                if (typeof element === 'number') {
                    container.style[key] = element + "px";
                }
                else {
                    container.style[key] = element;
                }
            }
        }
        this._hostElement.appendChild(container);
        window.getComputedStyle(container).getPropertyValue('opacity');
        container.style.transform = "scale(1)";
    };
    Ripple.prototype.onPointerDown = function (event) {
        if (!this.rippleConfig.disabled) {
            /**Destroy previous ripple if exist */
            this.endRipple();
            this.startRipple(event, this.rippleConfig);
        }
    };
    Ripple.prototype.onPointerLeave = function (event) {
        if (!this.rippleConfig.disabled) {
            this.endRipple();
        }
    };
    Ripple.prototype.startRipple = function (event, rippleConfig) {
        var containerRect = this._rectContainer;
        var x = event.clientX, y = event.clientY;
        if (rippleConfig.centered) {
            x = containerRect.left + containerRect.width / 2;
            y = containerRect.top + containerRect.height / 2;
        }
        var left = x - containerRect.left;
        var top = y - containerRect.top;
        var radius = rippleConfig.radius === 'containerSize' ? maxSize(containerRect) / 2 : rippleConfig.radius || rippleRadius(x, y, containerRect);
        if (rippleConfig.percentageToIncrease) {
            radius += radius * rippleConfig.percentageToIncrease / 100;
        }
        var ripple = this.createRipple({
            left: left - radius,
            top: top - radius,
            width: radius * 2,
            height: radius * 2,
            transitionDuration: this._transitionDuration
        });
    };
    Ripple.prototype.runTimeoutOutsideZone = function (fn, delay) {
        if (delay === void 0) { delay = 0; }
        this._ngZone.runOutsideAngular(function () { return setTimeout(fn, delay); });
    };
    Ripple.prototype.endRipple = function () {
        var rippleRef = this._rippleRef || null;
        var duration = parseFloat(this._transitionDuration);
        if (rippleRef && rippleRef.state) {
            rippleRef.end();
            this.runTimeoutOutsideZone(function () {
                rippleRef.container.style.opacity = '0';
                // }, rippleRef.timestamp < duration ? duration : 0);
            }, rippleRef.timestamp < duration ? duration / (duration * .001 + 1) : 0);
            this.runTimeoutOutsideZone(function () {
                rippleRef.container.parentNode.removeChild(rippleRef.container);
                // }, rippleRef.timestamp < duration ? duration * 2 : duration);
            }, rippleRef.timestamp < duration ? duration / (duration * .001 + 1) * 2 : duration);
        }
    };
    return Ripple;
}());
function rippleRadius(x, y, rect) {
    var distX = Math.max(Math.abs(x - rect.left), Math.abs(x - rect.right));
    var distY = Math.max(Math.abs(y - rect.top), Math.abs(y - rect.bottom));
    return Math.sqrt(distX * distX + distY * distY);
}
function maxSize(rect) {
    return Math.max(rect.width, rect.height);
}

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var LyRipple = /** @class */ (function () {
    function LyRipple(_platformId, _elementRef, _ngZone, _renderer) {
        this._platformId = _platformId;
        this._elementRef = _elementRef;
        this._ngZone = _ngZone;
        this._renderer = _renderer;
        if (core.Platform.isBrowser) {
            this._containerElement = this._elementRef.nativeElement;
        }
    }
    Object.defineProperty(LyRipple.prototype, "lyRippleConfig", {
        get: function () {
            return {
                centered: this.lyRippleCentered,
                disabled: this.lyRippleDisabled,
                sensitive: this.lyRippleSensitive,
                radius: this.lyRippleRadius,
                percentageToIncrease: this.lyRipplePercentageToIncrease,
            };
        },
        enumerable: true,
        configurable: true
    });
    LyRipple.prototype.ngOnInit = function () {
        this._updateRipple();
    };
    LyRipple.prototype.ngOnChanges = function (changes) {
        this._updateRipple();
    };
    LyRipple.prototype._updateRipple = function () {
        if (core.Platform.isBrowser) {
            if (this.lyRippleConfig.disabled) {
                return;
            }
            if (!this.rippleContainer) {
                this.rippleContainer = new Ripple(this._containerElement, this._renderer, this._ngZone);
            }
            this.rippleContainer.rippleConfig = this.lyRippleConfig;
        }
    };
    /**
     * Use only in ngAfterViewInit
     */
    /**
       * Use only in ngAfterViewInit
       */
    LyRipple.prototype.setTriggerElement = /**
       * Use only in ngAfterViewInit
       */
    function (triggerElement) {
        this.lyRippleDisabled = true;
        this.rippleContainer.setTriggerElement(triggerElement);
        this.lyRippleDisabled = false;
    };
    LyRipple.prototype.ngOnDestroy = function () {
        if (this.rippleContainer) {
            this.rippleContainer.setTriggerElement(null);
        }
    };
    LyRipple.decorators = [
        { type: core$1.Directive, args: [{
                    selector: '[lyRipple]',
                    exportAs: 'lyRipple'
                },] },
    ];
    /** @nocollapse */
    LyRipple.ctorParameters = function () { return [
        { type: Object, decorators: [{ type: core$1.Inject, args: [core$1.PLATFORM_ID,] },] },
        { type: core$1.ElementRef, },
        { type: core$1.NgZone, },
        { type: core$1.Renderer2, },
    ]; };
    LyRipple.propDecorators = {
        "lyRippleCentered": [{ type: core$1.Input },],
        "lyRippleDisabled": [{ type: core$1.Input },],
        "lyRippleSensitive": [{ type: core$1.Input },],
        "lyRippleRadius": [{ type: core$1.Input },],
        "lyRipplePercentageToIncrease": [{ type: core$1.Input },],
    };
    __decorate([
        core.IsBoolean(),
        __metadata("design:type", Boolean)
    ], LyRipple.prototype, "lyRippleCentered", void 0);
    __decorate([
        core.IsBoolean(),
        __metadata("design:type", Boolean)
    ], LyRipple.prototype, "lyRippleDisabled", void 0);
    __decorate([
        core.IsBoolean(),
        __metadata("design:type", Boolean)
    ], LyRipple.prototype, "lyRippleSensitive", void 0);
    return LyRipple;
}());

var LyRippleModule = /** @class */ (function () {
    function LyRippleModule() {
    }
    LyRippleModule.decorators = [
        { type: core$1.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    declarations: [LyRipple],
                    exports: [LyRipple]
                },] },
    ];
    /** @nocollapse */
    LyRippleModule.ctorParameters = function () { return []; };
    return LyRippleModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

exports.LyRippleModule = LyRippleModule;
exports.LyRipple = LyRipple;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=alyle-ui__ripple-minimal.umd.js.map
