import { Directive, Input, ElementRef, HostBinding, Renderer2 } from '@angular/core';
import { LyShadowService } from './shadow.service';
import { LyTheme } from 'alyle-ui/core';
export class LyDeepComponent {
    constructor(elementRef, renderer, theme, shadow) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.theme = theme;
        this.shadow = shadow;
        this._size = 1;
        this._interval = 3000;
        this.numberState = -1;
        this._shadowColor = 'rgba(0,0,0,0.87)';
        /** Default elevation */
        this.elevation = 1;
    }
    set lyShadow(val) {
        this.shadow.setShadow(this.elementRef, this.renderer, val, this._lastClass);
    }
    set shadowColor(val) {
        this._shadowColor = this.theme.colorOf(val);
        this.styleHost();
    }
    get shadowColor() {
        return this._shadowColor;
    }
    styleHost() {
        this.styleBoxShadow = this.shadow.shadow(this.shadowColor, this.scale);
    }
    get scale() {
        return this._size;
    }
    set scale(val) {
        this._size = val;
        this.styleHost();
    }
}
LyDeepComponent.decorators = [
    { type: Directive, args: [{
                selector: '[ly-deep], [ly-shadow], [lyShadow]'
            },] },
];
/** @nocollapse */
LyDeepComponent.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
    { type: LyTheme, },
    { type: LyShadowService, },
];
LyDeepComponent.propDecorators = {
    "styleBoxShadow": [{ type: HostBinding, args: ['style.box-shadow',] },],
    "lyShadow": [{ type: Input },],
    "shadowColor": [{ type: Input, args: ['shadowColor',] },],
    "scale": [{ type: Input },],
};
//# sourceMappingURL=deep.js.map