(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common/http'), require('rxjs/add/observable/of'), require('rxjs/add/operator/map'), require('alyle-ui/core'), require('@angular/common')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common/http', 'rxjs/add/observable/of', 'rxjs/add/operator/map', 'alyle-ui/core', '@angular/common'], factory) :
	(factory((global.alyleUi = global.alyleUi || {}, global.alyleUi.svg = {}),global.ng.core,global.ng.common.http,global.Rx.Observable.prototype,global.Rx.Observable.prototype,global.ly.core,global.ng.common));
}(this, (function (exports,core,http,of,map,core$1,common) { 'use strict';

var LySvgService = /** @class */ (function () {
    function LySvgService(_http) {
        this._http = _http;
    }
    LySvgService.prototype.getSVG = function (url) {
        var _this = this;
        if (LySvgService._svg.has(url)) {
            return Promise.resolve(this._cloneSVG(LySvgService._svg.get(url)));
        }
        var req = this._http.get(url, { responseType: 'text' })
            .map(function (res) {
            var svgEl = _this._svgElementFromString(res);
            LySvgService._svg.set(url, svgEl);
            return _this._cloneSVG(svgEl);
        })
            .toPromise();
        return req;
    };
    LySvgService.prototype._svgElementFromString = function (str) {
        var div = document.createElement('DIV');
        div.innerHTML = str;
        var svg = div.querySelector('svg');
        if (!svg) {
            throw new Error('No SVG found in loaded contents');
        }
        return svg;
    };
    LySvgService.prototype._cloneSVG = function (svg) {
        return svg.cloneNode(true);
    };
    LySvgService._svg = new Map();
    LySvgService.decorators = [
        { type: core.Injectable },
    ];
    /** @nocollapse */
    LySvgService.ctorParameters = function () { return [
        { type: http.HttpClient, },
    ]; };
    return LySvgService;
}());

var LySvgComponent = /** @class */ (function () {
    function LySvgComponent(svgService, _elementRef) {
        this.svgService = svgService;
        this._elementRef = _elementRef;
        this._size = '48px';
        this.styleWidth = '24px';
        this.styleHeight = '24px';
        this.prepend = true;
    }
    Object.defineProperty(LySvgComponent.prototype, "src", {
        get: function () {
            return this._src;
        },
        set: function (value) {
            if (!this._strEndsWith(value, '.svg')) {
                this._src = value + ".svg";
            }
            else {
                this._src = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LySvgComponent.prototype, "size", {
        get: function () {
            return this._size;
        },
        set: function (val) {
            if (typeof val === 'number') {
                this._size = val + "px";
            }
            else {
                this._size = val;
            }
            this.updateSize();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(LySvgComponent.prototype, "sizePx", {
        set: function (value) {
            if (value !== this._size) {
                this._size = value + "px";
                this.updateSize();
            }
        },
        enumerable: true,
        configurable: true
    });
    LySvgComponent.prototype.updateSize = function () {
        this.styleWidth = this.size;
        this.styleHeight = this.size;
    };
    LySvgComponent.prototype._strEndsWith = function (str, suffix) {
        return str.match(suffix + '$') == suffix;
    };
    LySvgComponent.prototype.ngOnInit = function () {
    };
    LySvgComponent.prototype.ngOnChanges = function (changes) {
        if (core$1.Platform.isBrowser) {
            if (changes['src']) {
                this._insertSVG();
            }
        }
    };
    LySvgComponent.prototype._insertSVG = function () {
        var _this = this;
        this.svgService.getSVG(this.src)
            .then(function (svg) {
            // console.log('suss src Insert SVG');
            // Insert SVG
            if (svg && _this._elementRef.nativeElement) {
                _this._elementRef.nativeElement.innerHTML = '';
                _this._insertElementSVG(svg);
            }
        }).catch(function (err) {
            console.warn('err', err);
        });
    };
    LySvgComponent.prototype._insertElementSVG = function (el) {
        if (this.prepend) {
            this._elementRef.nativeElement.insertBefore(el, this._elementRef.nativeElement.firstChild);
        }
        else {
            this._elementRef.nativeElement.appendChild(el);
        }
    };
    LySvgComponent.decorators = [
        { type: core.Directive, args: [{
                    selector: 'ly-svg, [ly-svg], [lySvg]'
                },] },
    ];
    /** @nocollapse */
    LySvgComponent.ctorParameters = function () { return [
        { type: LySvgService, },
        { type: core.ElementRef, },
    ]; };
    LySvgComponent.propDecorators = {
        "src": [{ type: core.Input, args: ['src',] },],
        "styleWidth": [{ type: core.HostBinding, args: ['style.width',] },],
        "styleHeight": [{ type: core.HostBinding, args: ['style.height',] },],
        "prepend": [{ type: core.Input },],
        "size": [{ type: core.Input, args: ['size',] },],
        "sizePx": [{ type: core.Input, args: ['size.px',] },],
    };
    return LySvgComponent;
}());

var LySvgModule = /** @class */ (function () {
    function LySvgModule() {
    }
    LySvgModule.forRoot = function () {
        return {
            ngModule: LySvgModule,
        };
    };
    LySvgModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [
                        common.CommonModule
                    ],
                    exports: [LySvgComponent],
                    declarations: [LySvgComponent],
                    providers: [LySvgService]
                },] },
    ];
    /** @nocollapse */
    LySvgModule.ctorParameters = function () { return []; };
    return LySvgModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

exports.LySvgModule = LySvgModule;
exports.LySvgService = LySvgService;
exports.LySvgComponent = LySvgComponent;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=alyle-ui__svg.umd.js.map
