import { Directive, Input, ElementRef, HostBinding, } from '@angular/core';
import { Platform } from 'alyle-ui/core';
import { LySvgService } from './ly-svg.service';
export class LySvgComponent {
    constructor(svgService, _elementRef) {
        this.svgService = svgService;
        this._elementRef = _elementRef;
        this._size = '48px';
        this.styleWidth = '24px';
        this.styleHeight = '24px';
        this.prepend = true;
    }
    set src(value) {
        if (!this._strEndsWith(value, '.svg')) {
            this._src = `${value}.svg`;
        }
        else {
            this._src = value;
        }
    }
    get src() {
        return this._src;
    }
    get size() {
        return this._size;
    }
    set size(val) {
        if (typeof val === 'number') {
            this._size = `${val}px`;
        }
        else {
            this._size = val;
        }
        this.updateSize();
    }
    set sizePx(value) {
        if (value !== this._size) {
            this._size = `${value}px`;
            this.updateSize();
        }
    }
    updateSize() {
        this.styleWidth = this.size;
        this.styleHeight = this.size;
    }
    _strEndsWith(str, suffix) {
        return str.match(suffix + '$') == suffix;
    }
    ngOnInit() {
    }
    ngOnChanges(changes) {
        if (Platform.isBrowser) {
            if (changes['src']) {
                this._insertSVG();
            }
        }
    }
    _insertSVG() {
        this.svgService.getSVG(this.src)
            .then((svg) => {
            // console.log('suss src Insert SVG');
            // Insert SVG
            if (svg && this._elementRef.nativeElement) {
                this._elementRef.nativeElement.innerHTML = '';
                this._insertElementSVG(svg);
            }
        }).catch((err) => {
            console.warn('err', err);
        });
    }
    _insertElementSVG(el) {
        if (this.prepend) {
            this._elementRef.nativeElement.insertBefore(el, this._elementRef.nativeElement.firstChild);
        }
        else {
            this._elementRef.nativeElement.appendChild(el);
        }
    }
}
LySvgComponent.decorators = [
    { type: Directive, args: [{
                selector: 'ly-svg, [ly-svg], [lySvg]'
            },] },
];
/** @nocollapse */
LySvgComponent.ctorParameters = () => [
    { type: LySvgService, },
    { type: ElementRef, },
];
LySvgComponent.propDecorators = {
    "src": [{ type: Input, args: ['src',] },],
    "styleWidth": [{ type: HostBinding, args: ['style.width',] },],
    "styleHeight": [{ type: HostBinding, args: ['style.height',] },],
    "prepend": [{ type: Input },],
    "size": [{ type: Input, args: ['size',] },],
    "sizePx": [{ type: Input, args: ['size.px',] },],
};
//# sourceMappingURL=ly-svg.component.js.map